from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.pool_historical_prepayments import PoolHistoricalPrepayments
from ...models.pool_historical_prepayments_error import PoolHistoricalPrepaymentsError
from ...models.pool_historical_prepayments_not_found import (
    PoolHistoricalPrepaymentsNotFound,
)
from ...models.pool_historical_prepayments_unauthorized import (
    PoolHistoricalPrepaymentsUnauthorized,
)
from ...models.pool_service_get_historical_prepayments_convention import (
    PoolService_GetHistoricalPrepaymentsConvention,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    cusips: List[str],
    periodyear: Union[Unset, int] = UNSET,
    periodmonth: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PoolService_GetHistoricalPrepaymentsConvention
    ] = PoolService_GetHistoricalPrepaymentsConvention.PERIOD,
    num_months: Union[Unset, int] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/pools/prepayment_history".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_cusips = cusips

    json_convention: Union[
        Unset, PoolService_GetHistoricalPrepaymentsConvention
    ] = UNSET
    if not isinstance(convention, Unset):
        json_convention = convention

    params: Dict[str, Any] = {
        "CUSIPS": json_cusips,
        "period.year": periodyear,
        "period.month": periodmonth,
        "convention": json_convention,
        "numMonths": num_months,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    if response.status_code == 200:
        response_200 = PoolHistoricalPrepayments.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = PoolHistoricalPrepaymentsUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = PoolHistoricalPrepaymentsNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = PoolHistoricalPrepaymentsError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    cusips: List[str],
    periodyear: Union[Unset, int] = UNSET,
    periodmonth: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PoolService_GetHistoricalPrepaymentsConvention
    ] = PoolService_GetHistoricalPrepaymentsConvention.PERIOD,
    num_months: Union[Unset, int] = UNSET,
) -> Response[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cusips=cusips,
        periodyear=periodyear,
        periodmonth=periodmonth,
        convention=convention,
        num_months=num_months,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    cusips: List[str],
    periodyear: Union[Unset, int] = UNSET,
    periodmonth: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PoolService_GetHistoricalPrepaymentsConvention
    ] = PoolService_GetHistoricalPrepaymentsConvention.PERIOD,
    num_months: Union[Unset, int] = UNSET,
) -> Optional[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    """


    Args:
        cusips: Cohort to fetch.
        periodyear: Year of the period. Must be from 1 to 9999, or 0 to specify a date without a year
        periodmonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
        convention: Convention to use for the date, can be PERIOD or FACTOR_DATE.
        num_months: Number of months of historical data. Default to 5

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        cusips=cusips,
        periodyear=periodyear,
        periodmonth=periodmonth,
        convention=convention,
        num_months=num_months,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    cusips: List[str],
    periodyear: Union[Unset, int] = UNSET,
    periodmonth: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PoolService_GetHistoricalPrepaymentsConvention
    ] = PoolService_GetHistoricalPrepaymentsConvention.PERIOD,
    num_months: Union[Unset, int] = UNSET,
) -> Response[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cusips=cusips,
        periodyear=periodyear,
        periodmonth=periodmonth,
        convention=convention,
        num_months=num_months,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    cusips: List[str],
    periodyear: Union[Unset, int] = UNSET,
    periodmonth: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PoolService_GetHistoricalPrepaymentsConvention
    ] = PoolService_GetHistoricalPrepaymentsConvention.PERIOD,
    num_months: Union[Unset, int] = UNSET,
) -> Optional[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    """  """

    return (
        await asyncio_detailed(
            client=client,
            cusips=cusips,
            periodyear=periodyear,
            periodmonth=periodmonth,
            convention=convention,
            num_months=num_months,
        )
    ).parsed
