from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.pool_predictions import PoolPredictions
from ...models.pool_predictions_bad_request import PoolPredictionsBadRequest
from ...models.pool_predictions_error import PoolPredictionsError
from ...models.pool_predictions_not_found import PoolPredictionsNotFound
from ...models.pool_predictions_unauthorized import PoolPredictionsUnauthorized
from ...models.prediction_service_get_pools_convention import (
    PredictionService_GetPoolsConvention,
)
from ...models.prediction_service_get_pools_scenario import (
    PredictionService_GetPoolsScenario,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    cusips: List[str],
    as_ofyear: Union[Unset, int] = UNSET,
    as_ofmonth: Union[Unset, int] = UNSET,
    as_ofday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetPoolsConvention
    ] = PredictionService_GetPoolsConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetPoolsScenario
    ] = PredictionService_GetPoolsScenario.BASELINE,
) -> Dict[str, Any]:
    url = "{}/api/v1/predictions/pools/latest".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_cusips = cusips

    json_convention: Union[Unset, PredictionService_GetPoolsConvention] = UNSET
    if not isinstance(convention, Unset):
        json_convention = convention

    json_scenario: Union[Unset, PredictionService_GetPoolsScenario] = UNSET
    if not isinstance(scenario, Unset):
        json_scenario = scenario

    params: Dict[str, Any] = {
        "CUSIPS": json_cusips,
        "asOf.year": as_ofyear,
        "asOf.month": as_ofmonth,
        "asOf.day": as_ofday,
        "convention": json_convention,
        "scenario": json_scenario,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    if response.status_code == 200:
        response_200 = PoolPredictions.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = PoolPredictionsBadRequest.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = PoolPredictionsUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = PoolPredictionsNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = PoolPredictionsError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    cusips: List[str],
    as_ofyear: Union[Unset, int] = UNSET,
    as_ofmonth: Union[Unset, int] = UNSET,
    as_ofday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetPoolsConvention
    ] = PredictionService_GetPoolsConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetPoolsScenario
    ] = PredictionService_GetPoolsScenario.BASELINE,
) -> Response[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cusips=cusips,
        as_ofyear=as_ofyear,
        as_ofmonth=as_ofmonth,
        as_ofday=as_ofday,
        convention=convention,
        scenario=scenario,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    cusips: List[str],
    as_ofyear: Union[Unset, int] = UNSET,
    as_ofmonth: Union[Unset, int] = UNSET,
    as_ofday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetPoolsConvention
    ] = PredictionService_GetPoolsConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetPoolsScenario
    ] = PredictionService_GetPoolsScenario.BASELINE,
) -> Optional[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    """
    Fetch individual pool prepayment predictions for an as-of date, default to today.

    Args:
        cusips: Pool CUSIP or CUSIPS to fetch individual prepayment predictions.
        as_ofyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
        as_ofmonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
        as_ofday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
        convention: Convention to use for the date, can be PERIOD or FACTOR_DATE.
        scenario: None

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        cusips=cusips,
        as_ofyear=as_ofyear,
        as_ofmonth=as_ofmonth,
        as_ofday=as_ofday,
        convention=convention,
        scenario=scenario,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    cusips: List[str],
    as_ofyear: Union[Unset, int] = UNSET,
    as_ofmonth: Union[Unset, int] = UNSET,
    as_ofday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetPoolsConvention
    ] = PredictionService_GetPoolsConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetPoolsScenario
    ] = PredictionService_GetPoolsScenario.BASELINE,
) -> Response[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cusips=cusips,
        as_ofyear=as_ofyear,
        as_ofmonth=as_ofmonth,
        as_ofday=as_ofday,
        convention=convention,
        scenario=scenario,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    cusips: List[str],
    as_ofyear: Union[Unset, int] = UNSET,
    as_ofmonth: Union[Unset, int] = UNSET,
    as_ofday: Union[Unset, int] = UNSET,
    convention: Union[
        Unset, PredictionService_GetPoolsConvention
    ] = PredictionService_GetPoolsConvention.PERIOD,
    scenario: Union[
        Unset, PredictionService_GetPoolsScenario
    ] = PredictionService_GetPoolsScenario.BASELINE,
) -> Optional[
    Union[
        PoolPredictions,
        PoolPredictionsBadRequest,
        PoolPredictionsUnauthorized,
        PoolPredictionsNotFound,
        PoolPredictionsError,
    ]
]:
    """ Fetch individual pool prepayment predictions for an as-of date, default to today. """

    return (
        await asyncio_detailed(
            client=client,
            cusips=cusips,
            as_ofyear=as_ofyear,
            as_ofmonth=as_ofmonth,
            as_ofday=as_ofday,
            convention=convention,
            scenario=scenario,
        )
    ).parsed
